@@ -2,21 +2,21 @@
 # this repository contains the full copyright notices and license terms.
 
 import datetime
-from dateutil.relativedelta import relativedelta
 import itertools
 
+from dateutil.relativedelta import relativedelta
 from sql import Null
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce
 
 from trytond.i18n import gettext
-from trytond.model import ModelView, Workflow, ModelSQL, fields, Unique
+from trytond.model import ModelSQL, ModelView, Unique, Workflow, fields
 from trytond.model.exceptions import AccessError
-from trytond.wizard import Wizard, StateView, StateTransition, Button
-from trytond.pyson import Not, Equal, Eval, Or, Bool, If
-from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond.tools import reduce_ids, grouped_slice
+from trytond.pyson import Bool, Equal, Eval, If, Not, Or
+from trytond.tools import grouped_slice, reduce_ids
+from trytond.transaction import Transaction
+from trytond.wizard import Button, StateTransition, StateView, Wizard
 
 from .exceptions import ForecastValidationError
 
@@ -28,45 +28,40 @@ class Forecast(Workflow, ModelSQL, ModelView):
     _states = {
         'readonly': Not(Equal(Eval('state'), 'draft')),
     }
-    _depends = ['state']
 
     warehouse = fields.Many2One(
         'stock.location', 'Location', required=True,
         domain=[('type', '=', 'warehouse')], states={
             'readonly': Or(Not(Equal(Eval('state'), 'draft')),
                 Bool(Eval('lines', [0]))),
-            },
-        depends=['state'])
+            })
     destination = fields.Many2One(
         'stock.location', 'Destination', required=True,
-        domain=[('type', 'in', ['customer', 'production'])], states=_states,
-        depends=_depends)
+        domain=[('type', 'in', ['customer', 'production'])], states=_states)
     from_date = fields.Date(
         "From Date", required=True,
         domain=[('from_date', '<=', Eval('to_date'))],
-        states=_states, depends=_depends + ['to_date'])
+        states=_states)
     to_date = fields.Date(
         "To Date", required=True,
         domain=[('to_date', '>=', Eval('from_date'))],
-        states=_states, depends=_depends + ['from_date'])
+        states=_states)
     lines = fields.One2Many(
-        'stock.forecast.line', 'forecast', 'Lines', states=_states,
-        depends=_depends)
+        'stock.forecast.line', 'forecast', 'Lines', states=_states)
     company = fields.Many2One(
         'company.company', 'Company', required=True, states={
             'readonly': Or(Not(Equal(Eval('state'), 'draft')),
                 Bool(Eval('lines', [0]))),
-            },
-        depends=['state'])
+            })
     state = fields.Selection([
             ('draft', "Draft"),
             ('done', "Done"),
             ('cancelled', "Cancelled"),
-            ], "State", readonly=True, select=True)
+            ], "State", readonly=True, select=True, sort=False)
     active = fields.Function(fields.Boolean('Active'),
         'get_active', searcher='search_active')
 
-    del _states, _depends
+    del _states
 
     @classmethod
     def __setup__(cls):
@@ -167,37 +162,43 @@ class Forecast(Workflow, ModelSQL, ModelView):
         return [('warehouse.rec_name',) + tuple(clause[1:])]
 
     @classmethod
-    def validate(cls, forecasts):
-        super(Forecast, cls).validate(forecasts)
-        for forecast in forecasts:
-            forecast.check_date_overlap()
+    def validate_fields(cls, forecasts, field_names):
+        super().validate_fields(forecasts, field_names)
+        cls.check_date_overlap(forecasts, field_names)
 
-    def check_date_overlap(self):
-        if self.state != 'done':
+    @classmethod
+    def check_date_overlap(cls, forecasts, field_names=None):
+        if field_names and not (field_names & {
+                    'from_date', 'to_date',
+                    'warehouse', 'destination',
+                    'company', 'state'}):
             return
         transaction = Transaction()
         connection = transaction.connection
-        self.__class__.lock()
-        forcast = self.__table__()
+        cls.lock()
+        table = cls.__table__()
         cursor = connection.cursor()
-        cursor.execute(*forcast.select(forcast.id,
-                where=(((forcast.from_date <= self.from_date)
-                        & (forcast.to_date >= self.from_date))
-                    | ((forcast.from_date <= self.to_date)
-                        & (forcast.to_date >= self.to_date))
-                    | ((forcast.from_date >= self.from_date)
-                        & (forcast.to_date <= self.to_date)))
-                & (forcast.warehouse == self.warehouse.id)
-                & (forcast.destination == self.destination.id)
-                & (forcast.company == self.company.id)
-                & (forcast.id != self.id)))
-        forecast_id = cursor.fetchone()
-        if forecast_id:
-            second = self.__class__(forecast_id[0])
-            raise ForecastValidationError(
-                gettext('stock_forecast.msg_forecast_date_overlap',
-                    first=self.rec_name,
-                    second=second.rec_name))
+        for forecast in forecasts:
+            if forecast.state != 'done':
+                continue
+            cursor.execute(*table.select(table.id,
+                    where=(((table.from_date <= forecast.from_date)
+                            & (table.to_date >= forecast.from_date))
+                        | ((table.from_date <= forecast.to_date)
+                            & (table.to_date >= forecast.to_date))
+                        | ((table.from_date >= forecast.from_date)
+                            & (table.to_date <= forecast.to_date)))
+                    & (table.warehouse == forecast.warehouse.id)
+                    & (table.destination == forecast.destination.id)
+                    & (table.company == forecast.company.id)
+                    & (table.id != forecast.id)))
+            forecast_id = cursor.fetchone()
+            if forecast_id:
+                second = cls(forecast_id[0])
+                raise ForecastValidationError(
+                    gettext('stock_forecast.msg_forecast_date_overlap',
+                        first=forecast.rec_name,
+                        second=second.rec_name))
 
     @classmethod
     def delete(cls, forecasts):
@@ -259,14 +260,13 @@ class ForecastLine(ModelSQL, ModelView):
     _states = {
         'readonly': Eval('forecast_state') != 'draft',
         }
-    _depends = ['forecast_state']
 
     product = fields.Many2One('product.product', 'Product', required=True,
         domain=[
             ('type', '=', 'goods'),
             ('consumable', '=', False),
             ],
-        states=_states, depends=_depends)
+        states=_states)
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
         'on_change_with_product_uom_category')
@@ -277,15 +277,15 @@ class ForecastLine(ModelSQL, ModelView):
                 ('category', '!=', -1)),
             ],
         states=_states,
-        depends=['product', 'product_uom_category'] + _depends)
+        depends={'product'})
     quantity = fields.Float(
         "Quantity", digits='uom', required=True,
         domain=[('quantity', '>=', 0)],
-        states=_states, depends=_depends)
+        states=_states)
     minimal_quantity = fields.Float(
         "Minimal Qty", digits='uom', required=True,
         domain=[('minimal_quantity', '<=', Eval('quantity'))],
-        states=_states, depends=['quantity'] + _depends)
+        states=_states)
     moves = fields.Many2Many('stock.forecast.line-stock.move',
         'line', 'move', 'Moves', readonly=True)
     forecast = fields.Many2One(
@@ -293,15 +293,14 @@ class ForecastLine(ModelSQL, ModelView):
         states={
             'readonly': ((Eval('forecast_state') != 'draft')
                 & Bool(Eval('forecast'))),
-            },
-        depends=['forecast_state'])
+            })
     forecast_state = fields.Function(
         fields.Selection('get_forecast_states', 'Forecast State'),
         'on_change_with_forecast_state')
     quantity_executed = fields.Function(fields.Float(
             "Quantity Executed", digits='uom'), 'get_quantity_executed')
 
-    del _states, _depends
+    del _states
 
     @classmethod
     def __setup__(cls):
@@ -509,12 +508,10 @@ class ForecastCompleteAsk(ModelView):
     __name__ = 'stock.forecast.complete.ask'
     from_date = fields.Date(
         "From Date", required=True,
-        domain=[('from_date', '<', Eval('to_date'))],
-        depends=['to_date'])
+        domain=[('from_date', '<', Eval('to_date'))])
     to_date = fields.Date(
         "To Date", required=True,
-        domain=[('to_date', '>', Eval('from_date'))],
-        depends=['from_date'])
+        domain=[('to_date', '>', Eval('from_date'))])
 
 
 class ForecastCompleteChoose(ModelView):
